@@ -226,11 +226,13 @@ void KNotification::activate(unsigned int action)
         emit action3Activated();
         break;
     }
+
+    // emitting activated() makes the Manager close all the active plugins
+    // which will deref() the KNotification object, which will result
+    // in closing the notification
     emit activated(action);
-    if (d->id != -1) {
-        deleteLater();
-    }
-    d->id = -2;
+
+    d->id = -1;
 }
 
 void KNotification::close()
@@ -239,10 +241,12 @@ void KNotification::close()
         KNotificationManager::self()->close(d->id);
     }
 
-    d->id = -2;
+    if (d->id == -1) {
+        d->id = -2;
+        emit closed();
+        deleteLater();
+    }
 
-    emit closed();
-    deleteLater();
 }
 
 void KNotification::raiseWidget()
@@ -331,6 +335,7 @@ void KNotification::deref()
 {
     d->ref--;
     if (d->ref == 0) {
+        d->id = -1;
         close();
     }
 }
